@@ -1,4 +1,5 @@
 //2: try more memory efficient
+//Sep. 27, 2014: Change BASIS_PHI to two dimensional pointer. Orignal developed for EBlasso version 3.2
 #include <R.h>
 #include <Rinternals.h>
 #include <math.h>
@@ -14,18 +15,19 @@ void fEBInitializationGfNeEN(double *Alpha, double * PHI, int *Used, int *Unused
 				int n, int *m, int kdim, double *beta);
 
 void MatrixInverseGfNeEN(double * a,int N);
-void CacheBPGfNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
+void CacheBPGfNeEN(double **BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
 				double *Targets,double *scales,int N,int K,int M, int M_full);
 
 void fEBLinearFullStatGfNeEN(double *beta,double * SIGMA,double *H, double *S_in, double * Q_in, double * S_out, 
-				double * Q_out,  double *BASIS, double * Scales, double *PHI, double *BASIS_PHI,
+				double * Q_out,  double *BASIS, double * Scales, double *PHI, double **BASIS_PHI,
 				double *BASIS_Targets, double * Targets, int * Used, double *Alpha, double * Mu, 
 				 double *Gamma,int *n, int *m, int* kdim, int *iteration,int *i_iter);
 
 void fEBDeltaMLGfNeEN(double *DeltaML, int *Action, double *AlphaRoot, int *anyToDelete,
 				int *Used, int * Unused, double * S_out, double * Q_out, double *Alpha,
 				double *a_lmabda, double *b_Alpha,
-				int m, int mBar);
+				int m, int mBar,
+				double *deltaLogMarginal, int *nu);//Block Coordinate Ascent Algorithm Parameters;
 
 void LinearFastEmpBayesGfNeEN(int *Used, double *Mu, double *SIGMA, double *H, double *Alpha, double *PHI,
 				double *BASIS, double * Targets, double *Scales, double *a_lambda,double *b_Alpha,
@@ -33,12 +35,12 @@ void LinearFastEmpBayesGfNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 
 
 
-int ActionAddGfNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
+int ActionAddGfNeEN(double **BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
 			double *beta, double* Alpha, double newAlpha, double*SIGMA, double*Mu, double*S_in,
 			double*Q_in, int nu, double*SIGMANEW, int M_full, int N, int K, int M);
 
 
-int ActionDelGfNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double*BASIS_PHI,
+int ActionDelGfNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double**BASIS_PHI,
 				double*Mu, double*S_in, double*Q_in, double *beta, int jj, int N, int M, int M_full);
 
 double varTargetsGfNeEN(double* Target,int N);
@@ -274,6 +276,12 @@ void LinearFastEmpBayesGfNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	const int		CNBetaUpdateStart	=10;
 	const double	BetaMaxFactor		=1e6;
 	const double	MinDeltaLogBeta		=1e-6;
+	
+	//Block Coordinate Ascent Algorithm Parameters;
+	const double 	nAdd 				= 0.99; //only deltaML >nAdd will be considered as candidate
+	double nAdd_real 					= nAdd;
+	const double 	MLdelta 			= 0.01; //only deltaML > MLdelta will be selected for action.
+	
     //[Alpha,PHI2,Used,Unused,Mu2]=InitialCategory(BASIS,Targets,Scales,PHI2,Used,Alpha,Mu2,IniLogic) 
     int *IniLogic;
 	IniLogic			= (int*) Calloc(1,int);
@@ -294,7 +302,7 @@ void LinearFastEmpBayesGfNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	//lapack
 	int inci =1;
 	int incj =1;
-	double *readPtr1, *readPtr2;
+	double *readPtr1;//, *readPtr2;
 	int inc0 = 0;
 	double a_blas = 1;
 	double b_blas = 1;
@@ -308,15 +316,25 @@ void LinearFastEmpBayesGfNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	//Rprintf("N_used is: %d; N_unused:%d, M: %d,sample size: %d \n",N_used,N_unused,M,N);
 
 	fEBInitializationGfNeEN(Alpha, PHI, Used, Unused, BASIS, Targets, Scales, IniLogic, N, m, K,beta);
+	
+	//Block Coordinate Ascent Algorithm Parameters;
+	M = m[0];
+	N_unused = M_full - M;
+	
 	if(verbose >3) Rprintf("\t Initialized basis %d, Alpha: %f, \n", Used[0],Alpha[0]);
 	//Rprintf("\t beta: %f\n",beta[0]);
 	//Rprintf("\t m: %d\n",M);
 	//for(i=0;i<N;i++) Rprintf("\tPHI: \t %f\n",PHI[i]);
 	//for(i=0;i<10;i++) Rprintf("PHI2: %f \t  %f; BASIS: %f\n",PHI2[i],PHI2[N+i],BASIS[181*N+i]/Scales[181]);
     //CACHE MATRIX
-	double *BASIS_Targets,*BASIS_PHI;
+	double *BASIS_Targets,**BASIS_PHI;/* ********* Sep. 27, 2014 ************* */
 	BASIS_Targets		= (double *) Calloc(M_full,double);
-	BASIS_PHI			= (double *) Calloc(M_full*basisMax,double);
+	BASIS_PHI			= (double **) Calloc(basisMax,double);
+	for(i=0;i<M;i++)
+	{
+		BASIS_PHI[i] 	= (double *) Calloc(M_full,double);
+	}
+	
 	CacheBPGfNeEN(BASIS_PHI, BASIS_Targets, BASIS, PHI,	Targets,Scales,N,K,M,M_full);
 	
 	double *S_in, *Q_in, *S_out, *Q_out,*gamma;
@@ -364,228 +382,282 @@ void LinearFastEmpBayesGfNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	
 	double temp;			// for action_reestimate
 	double * SIGMANEW	= (double * ) Calloc(basisMax*basisMax, double);
-if(verbose >3) Rprintf("check point 3: before loop \n");
+	
+	//Block Coordinate Ascent Algorithm Parameters;
+	double cutoff;
+	int *nuAction;
+	nuAction = (int*) Calloc(M_full,int);
+	int nUpdate, iU;// nTerminate;
+	
+	if(verbose>3 && iter ==1)
+	{
+		Rprintf("check point 3: before loop,initial number of basis:%d\t Basis initialized: ",M);
+		for(i=0;i<M;i++) Rprintf("%dth:%d\t",i,Used[i]);
+		Rprintf("\n");
+		
+	}
    while(LAST_ITERATION!=1)
     {
         i_iter						= i_iter + 1;
 
-		if(verbose >4) Rprintf("\t inner loop %d \n",i_iter);
+		if(verbose >4) Rprintf("\t inner loop %d; number of basis: %d \n",i_iter, M);
         //M							= N_used + 1;
      	//N_unused					= M_full - N_used;
 
         //[DeltaML,Action,AlphaRoot,anyToDelete]     = fEBDeltaMLGfNeEN(Used,Unused,S_out,Q_out,Alpha,a,b);
 		fEBDeltaMLGfNeEN(DeltaML, Action, AlphaRoot,anyToDelete,Used, Unused, S_out, Q_out, Alpha,
-				a_lambda, b_Alpha,M, N_unused);
+				a_lambda, b_Alpha,M, N_unused,
+				&deltaLogMarginal,&nu);
 		//
-        deltaLogMarginal			= 0;
-        nu							= -1;
-        for(i=0;i<M_full;i++)
-        {
-            if(DeltaML[i]>deltaLogMarginal)
-            {
-                deltaLogMarginal    = DeltaML[i];
-                nu                  = i;
-            }
-        }
+        //deltaLogMarginal			= MLdelta;
+        //nu							= -1;
+        //for(i=0;i<M_full;i++)
+        //{
+        //    if(DeltaML[i]>deltaLogMarginal)
+        //    {
+         //       deltaLogMarginal    = DeltaML[i];
+         //       nu                  = i;
+        //    }
+        //}
         // selectedAction          = Action[nu];
         
         if(nu==-1)
 		{		
 			anyWorthwhileAction     = 0;
-			selectedAction          = -10;
+			selectedAction          = ACTION_TERMINATE;
 		}else
 		{
 			anyWorthwhileAction	= 1;
-			selectedAction          = Action[nu];
-			newAlpha                = AlphaRoot[nu];
-		}	
-//		Rprintf("\t ActionOn nu= %d, deltaML: %f, selectedAction: %d, Alpha: %f\n",nu+1, DeltaML[nu],selectedAction,AlphaRoot[nu]);
-        if(selectedAction==ACTION_REESTIMATE || selectedAction==ACTION_DELETE)
-        {
-            index                   = nu + 1; 
-            for(i=0;i<M;i++)
-            {
-                if (Used[i]==index)	
-				{
-						jj  = i;
-						break;
-				}
-            }
-        }
-
-        kk                          = K;                          
-        for(i=0;i<K;i++)
-        {
-            if (i==nu)
-            {
-                //for(L=0;L<N;L++)    phi[L]  = BASIS[i*N+L]/Scales[i];
-				readPtr1 		= &BASIS[i*N];
-				F77_CALL(dcopy)(&N,readPtr1,&inci,phi,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
-				b_blas 			= 1/Scales[i];
-				F77_CALL(dscal)(&N,&b_blas,phi,&inci); 		//dscal(n, a, x, incx) x = a*x
-				
-				//break;
-            }
-			else if(i<(K-1))
-            {
-                for(j=(i+1);j<K;j++)
-                {
-                    if(kk==nu)
-                    {
-                        for(L=0;L<N;L++)    phi[L] =BASIS[i*N+L]*BASIS[j*N+L]/Scales[kk];
-						//break;
-                    }
-                    kk              = kk + 1;
-                }
-            }
-        }
-//		Rprintf("N: %d, kk: %d,i: %d, j: %d, K: %d\n",N,kk,i,j,K);
-		//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
-        //newAlpha                    = AlphaRoot[nu];
-//		Rprintf("\tNew alpha: %f\n",newAlpha);
-        if(anyWorthwhileAction==0)  selectedAction = ACTION_TERMINATE;
-        if(selectedAction==ACTION_REESTIMATE)
-        {
-            if (fabs(log(newAlpha)-log(Alpha[jj]))<=0.1 && anyToDelete[0] ==0)
-            {	
-    			//cout<<"Terminated by small deltaAlpha on basis:"<<nu<<endl;
-                selectedAction		= ACTION_TERMINATE;
-            }
-        }
-//Rprintf("\t selectedAction: %d\n",selectedAction);
-        //
-        UPDATE_REQUIRED				= 0;
-        if(selectedAction==ACTION_REESTIMATE)
-        {
-			if(verbose >4) Rprintf("\t\t Action: Reestimate : %d \t deltaML: %f\n",nu + 1, deltaLogMarginal);
-            oldAlpha				= Alpha[jj];
-            Alpha[jj]				= newAlpha;
-
-            deltaInv				= 1.0/(newAlpha-oldAlpha);
-            kappa					= 1.0/(SIGMA[jj*M+jj] + deltaInv);
-            Mujj					= Mu[jj];
-            //for(i=0;i<M;i++)		Mu[i]    = Mu[i] - Mujj *kappa * SIGMA[jj*M+i];
-			readPtr1 				= &SIGMA[jj*M];
-			b_blas 					= -Mujj * kappa;
-			F77_CALL(daxpy)(&M, &b_blas,readPtr1, &inci,Mu, &incj); //daxpy(n, a, x, incx, y, incy) y := a*x + y
-
- 			//	SIGMANEW	=SIGMA-sjj;
-			for(i=0;i<M;i++)
+			//Block Coordinate Ascent Algorithm Parameters;
+			// find a block to update;
+			if(Action[nu] == ACTION_ADD)
 			{
-				for(j=0;j<M;j++)	SIGMANEW[j*M + i] = SIGMA[j*M + i] - kappa * SIGMA[jj*M+i]*SIGMA[jj*M+j];
+				nAdd_real			= nAdd;				
+			}else
+			{
+				nAdd_real 			= 1;
+
 			}
-			
-			//S_in		= S_in + kappa*(beta*BASIS_PHI * s_j).^2;
-			//Q_in		= Q_in - beta*BASIS_PHI*(deltaMu);           
-			
+			cutoff 					= deltaLogMarginal * nAdd_real;
+			if(cutoff< MLdelta)
+			{
+				cutoff 				= MLdelta;
+			}
+			nUpdate 				= 0;
 			for(i=0;i<M_full;i++)
 			{
-				temp	= 0;
-				//for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
-				readPtr1 			= &BASIS_PHI[i];
-				readPtr2 			= &SIGMA[jj*M];
-				temp = F77_CALL(ddot)(&M, readPtr1, &M_full,readPtr2, &inci);
-				S_in[i]				= S_in[i] +  pow(beta[0]*temp,2)*kappa;
-				Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
+
+				if(DeltaML[i]>=cutoff)
+				{
+					nuAction[nUpdate]= i;
+					nUpdate++;
+				}
+				
 			}
-			//
-			//for(i=0;i<M_full;i++)
-			//{
-			//	temp	= 0;
-			//	for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]* SIGMA[jj*M+j];
-			//	Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
-			//}
-			UPDATE_REQUIRED			= 1;
-        }
-        /////////////////////////////////////////////////////////////////////////////////
-        else if(selectedAction==ACTION_ADD)
-        {
-			if(verbose >4) Rprintf("\t\t Action:add : %d \t deltaML: %f\n",nu + 1,deltaLogMarginal);
-			//Rprintf("\t\t newAlpha: %f\n",newAlpha);
-            // B_phi*PHI2*SIGMA2        tmp = B_phi*PHI2 
-            index					= M + 1;
-			if(index > (basisMax -10) && iter>1) {
-				Rprintf("bases: %d, warning: out of Memory!\n",index);
-			}//return;
-			if(index > (basisMax -1) && iter>1) {
-				Rprintf("bases: %d, out of Memory,exiting program!\n",index);
-				//exit(1);			
-			}
-			//BASIS_B_Phi	=BASIS_Phi*beta;
-			UPDATE_REQUIRED		= ActionAddGfNeEN(BASIS_PHI, BASIS, Scales, PHI, phi, beta, Alpha,
-				newAlpha, SIGMA, Mu, S_in, Q_in, nu, SIGMANEW, M_full, N, K, M);
+			if(Action[nu] == ACTION_DELETE && nUpdate>1) nUpdate = 1; // when deleting co-linear basis: delete one at a time
+			//if(verbose >4) Rprintf("\t deltaML cutoff: %f ; number to be updated: %d \n",cutoff, nUpdate);
+			
 
-            //			
-            Used[M]			= nu + 1;						//new element
+			for(iU=0;iU<nUpdate;iU++) //update each basis
+			{
+				nu 					= nuAction[iU];
+								
+				selectedAction      = Action[nu];
+				newAlpha            = AlphaRoot[nu];
+				deltaLogMarginal 	= DeltaML[nu];
+	
+		//		Rprintf("\t ActionOn nu= %d, deltaML: %f, selectedAction: %d, Alpha: %f\n",nu+1, DeltaML[nu],selectedAction,AlphaRoot[nu]);
+				if(selectedAction==ACTION_REESTIMATE || selectedAction==ACTION_DELETE)
+				{
+					index                   = nu + 1; 
+					for(i=0;i<M;i++)
+					{
+						if (Used[i]==index)	
+						{
+								jj  = i;
+								break;
+						}
+					}
+				}
 
-            //
+				kk                          = K;                          
+				for(i=0;i<K;i++)
+				{
+					if (i==nu)
+					{
+						//for(L=0;L<N;L++)    phi[L]  = BASIS[i*N+L]/Scales[i];
+						readPtr1 		= &BASIS[i*N];
+						F77_CALL(dcopy)(&N,readPtr1,&inci,phi,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+						b_blas 			= 1/Scales[i];
+						F77_CALL(dscal)(&N,&b_blas,phi,&inci); 		//dscal(n, a, x, incx) x = a*x
+						
+						//break;
+					}
+					else if(i<(K-1))
+					{
+						for(j=(i+1);j<K;j++)
+						{
+							if(kk==nu)
+							{
+								for(L=0;L<N;L++)    phi[L] =BASIS[i*N+L]*BASIS[j*N+L]/Scales[kk];
+								//break;
+							}
+							kk              = kk + 1;
+						}
+					}
+				}
+		//		Rprintf("N: %d, kk: %d,i: %d, j: %d, K: %d\n",N,kk,i,j,K);
+				//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
+				//newAlpha                    = AlphaRoot[nu];
+		//		Rprintf("\tNew alpha: %f\n",newAlpha);
+				if(anyWorthwhileAction==0)  selectedAction = ACTION_TERMINATE;
+				if(selectedAction==ACTION_REESTIMATE)
+				{
+					if (fabs(log(newAlpha)-log(Alpha[jj]))<=0.1 && anyToDelete[0] ==0)
+					{	
+						//cout<<"Terminated by small deltaAlpha on basis:"<<nu<<endl;
+						selectedAction		= ACTION_TERMINATE;
+					}
+				}
+				//Rprintf("\t selectedAction: %d\n",selectedAction);
+				//
+				UPDATE_REQUIRED				= 0;
+				if(selectedAction==ACTION_REESTIMATE)
+				{
+					if(verbose >4) Rprintf("\t\t Action: Reestimate : %d \t deltaML: %f\n",nu + 1, deltaLogMarginal);
+					oldAlpha				= Alpha[jj];
+					Alpha[jj]				= newAlpha;
 
-			N_unused				= N_unused - 1;
-			for(i=0;i<N_unused;i++)
-            {                
-                if(Unused[i]== (nu + 1))		Unused[i] =Unused[N_unused];
-            }
-			m[0]					= M + 1;
-			M						= m[0];
-			//for(i=0;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i],Mu[i]);
-			//Rprintf(" \t\t S_in[73]: %f, Q_in[73]: %f \n",S_in[nu],Q_in[nu]);
-			//for(i=0;i<4;i++) Rprintf(" \t\t SIGMANEW: %f \n",SIGMANEW[i]);
-			//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
-        }
-		//
-        else if(selectedAction==ACTION_DELETE)
-        {
-			if(verbose >4) Rprintf("\t\t Action: delete : %d deltaML: %f \n",nu + 1,deltaLogMarginal);
-            UPDATE_REQUIRED = ActionDelGfNeEN(PHI, Alpha, SIGMA, SIGMANEW, BASIS_PHI,
-				Mu, S_in, Q_in, beta, jj, N, M, M_full);
-			index					= M -1;
-            //Used; Unused;
-            Used[jj]				= Used[index];
+					deltaInv				= 1.0/(newAlpha-oldAlpha);
+					kappa					= 1.0/(SIGMA[jj*M+jj] + deltaInv);
+					Mujj					= Mu[jj];
+					//for(i=0;i<M;i++)		Mu[i]    = Mu[i] - Mujj *kappa * SIGMA[jj*M+i];
+					readPtr1 				= &SIGMA[jj*M];
+					b_blas 					= -Mujj * kappa;
+					F77_CALL(daxpy)(&M, &b_blas,readPtr1, &inci,Mu, &incj); //daxpy(n, a, x, incx, y, incy) y := a*x + y
 
-            //
-			N_unused				= N_unused + 1;
-			Unused[N_unused -1]		= nu + 1;
+					//	SIGMANEW	=SIGMA-sjj;
+					for(i=0;i<M;i++)
+					{
+						for(j=0;j<M;j++)	SIGMANEW[j*M + i] = SIGMA[j*M + i] - kappa * SIGMA[jj*M+i]*SIGMA[jj*M+j];
+					}
+					
+					//S_in		= S_in + kappa*(beta*BASIS_PHI * s_j).^2;
+					//Q_in		= Q_in - beta*BASIS_PHI*(deltaMu);           
+					
+					for(i=0;i<M_full;i++)
+					{
+						temp	= 0;
+						//for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
+						//readPtr1 			= &BASIS_PHI[i];
+						//readPtr2 			= &SIGMA[jj*M];
+						//temp = F77_CALL(ddot)(&M, readPtr1, &M_full,readPtr2, &inci);
+						for(j=0;j<M;j++) temp = temp + BASIS_PHI[j][i]*SIGMA[jj*M + j];
+						S_in[i]				= S_in[i] +  pow(beta[0]*temp,2)*kappa;
+						Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
+					}
+					//
+					//for(i=0;i<M_full;i++)
+					//{
+					//	temp	= 0;
+					//	for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]* SIGMA[jj*M+j];
+					//	Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
+					//}
+					UPDATE_REQUIRED			= 1;
+				}
+				/////////////////////////////////////////////////////////////////////////////////
+				else if(selectedAction==ACTION_ADD)
+				{
+					if(verbose >4) Rprintf("\t\t Action:add : %d \t deltaML: %f\n",nu + 1,deltaLogMarginal);
+					//Rprintf("\t\t newAlpha: %f\n",newAlpha);
+					// B_phi*PHI2*SIGMA2        tmp = B_phi*PHI2 
+					index					= M + 1;
+					if(index > (basisMax -10) && iter>1) {
+						Rprintf("bases: %d, warning: out of Memory!\n",index);
+					}//return;
+					if(index > (basisMax -1) && iter>1) {
+						Rprintf("bases: %d, out of Memory,exiting program!\n",index);
+						//exit(1);			
+					}
+					//BASIS_B_Phi	=BASIS_Phi*beta;
+					UPDATE_REQUIRED		= ActionAddGfNeEN(BASIS_PHI, BASIS, Scales, PHI, phi, beta, Alpha,
+						newAlpha, SIGMA, Mu, S_in, Q_in, nu, SIGMANEW, M_full, N, K, M);
 
-			m[0]					= M -1;
-			M						= m[0];
-        //for(i=1;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i-1],Mu2[i]);
+					//			
+					Used[M]			= nu + 1;						//new element
+
+					//
+
+					N_unused				= N_unused - 1;
+					for(i=0;i<N_unused;i++)
+					{                
+						if(Unused[i]== (nu + 1))		Unused[i] =Unused[N_unused];
+					}
+					m[0]					= M + 1;
+					M						= m[0];
+					//for(i=0;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i],Mu[i]);
+					//Rprintf(" \t\t S_in[73]: %f, Q_in[73]: %f \n",S_in[nu],Q_in[nu]);
+					//for(i=0;i<4;i++) Rprintf(" \t\t SIGMANEW: %f \n",SIGMANEW[i]);
+					//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
+				}
+				//
+				else if(selectedAction==ACTION_DELETE)
+				{
+					if(verbose >4) Rprintf("\t\t Action: delete : %d deltaML: %f \n",nu + 1,deltaLogMarginal);
+					UPDATE_REQUIRED = ActionDelGfNeEN(PHI, Alpha, SIGMA, SIGMANEW, BASIS_PHI,
+						Mu, S_in, Q_in, beta, jj, N, M, M_full);
+					index					= M -1;
+					//free deleted row of BASIS_PHI
+					Free(BASIS_PHI[index]);
+					//Used; Unused;
+					Used[jj]				= Used[index];
+
+					//
+					N_unused				= N_unused + 1;
+					Unused[N_unused -1]		= nu + 1;
+
+					m[0]					= M -1;
+					M						= m[0];
+				//for(i=1;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i-1],Mu2[i]);
+				}
+
+				//Rprintf("\t\t Update_required: %d\n",UPDATE_REQUIRED);
+				//
+				if(UPDATE_REQUIRED==1)
+				{
+					//for(i=0;i<M_full;i++)
+					//{
+					//	S_out[i]	= S_in[i];
+					//	Q_out[i]	= Q_in[i];
+					//}
+					F77_CALL(dcopy)(&M_full,S_in,&inci,S_out,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+					F77_CALL(dcopy)(&M_full,Q_in,&inci,Q_out,&incj); 
+					
+					for(i=0;i<M;i++)
+					{
+						index					= Used[i] -1;
+						S_out[index]			= Alpha[i]*S_in[index]/(Alpha[i]-S_in[index]);
+						Q_out[index]			= Alpha[i]*Q_in[index]/(Alpha[i]-S_in[index]);
+					}
+					//for(i=0;i<M;i++)
+					//{
+					//	for(j=0;j<M;j++) SIGMA[i*M+j]	= SIGMANEW[i*M+j];
+					//}
+					MM = M*M;			
+					F77_CALL(dcopy)(&MM,SIGMANEW,&inci,SIGMA,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+
+					for(i=0;i<M;i++) gamma[i]		= 1- Alpha[i]*SIGMA[i*M+i];
+				}
+			}//end of for(iU)
 		}
 
-        //Rprintf("\t\t Update_required: %d\n",UPDATE_REQUIRED);
-        //
-		if(UPDATE_REQUIRED==1)
-        {
-			//for(i=0;i<M_full;i++)
-			//{
-			//	S_out[i]	= S_in[i];
-			//	Q_out[i]	= Q_in[i];
-			//}
-			F77_CALL(dcopy)(&M_full,S_in,&inci,S_out,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
-			F77_CALL(dcopy)(&M_full,Q_in,&inci,Q_out,&incj); 
-			
-			for(i=0;i<M;i++)
-		    {
-				index					= Used[i] -1;
-				S_out[index]			= Alpha[i]*S_in[index]/(Alpha[i]-S_in[index]);
-				Q_out[index]			= Alpha[i]*Q_in[index]/(Alpha[i]-S_in[index]);
-			}
-			//for(i=0;i<M;i++)
-			//{
-			//	for(j=0;j<M;j++) SIGMA[i*M+j]	= SIGMANEW[i*M+j];
-			//}
-			MM = M*M;			
-			F77_CALL(dcopy)(&MM,SIGMANEW,&inci,SIGMA,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
-
-			for(i=0;i<M;i++) gamma[i]		= 1- Alpha[i]*SIGMA[i*M+i];
-        }
-
-		if ((selectedAction==ACTION_TERMINATE)||(i_iter<=CNBetaUpdateStart)||(i_iter%5==0))
+		if ((selectedAction==ACTION_TERMINATE)||(i_iter<=CNBetaUpdateStart)||(i_iter%5==0) || (nUpdate>=2))
 		{	
 			ee					= 0;
 			 //for(i=0;i<N;i++)
-            //{
-            //    PHI_Mu[i]       = 0;
-            //    for(j=0;j<M;j++)	PHI_Mu[i]   = PHI_Mu[i] + PHI[j*N+i]*Mu[j];
+			//{
+			//    PHI_Mu[i]       = 0;
+			//    for(j=0;j<M;j++)	PHI_Mu[i]   = PHI_Mu[i] + PHI[j*N+i]*Mu[j];
 				//e[i]			= Targets[i] - PHI_Mu[i];
 				//ee				= ee + e[i]*e[i];
 			 //}
@@ -626,12 +698,12 @@ if(verbose >3) Rprintf("check point 3: before loop \n");
 				}
 			 }
 			 //Rprintf("\tend of loop values N_used: %d, N_unused %d\n", M,N_unused);
-        }
+		}
 
 		//Rprintf("\t\t selected Action: %d\n",selectedAction);
 		//
         if(selectedAction==ACTION_TERMINATE) LAST_ITERATION =1;
-        if(i_iter==2*K)   LAST_ITERATION = 1;
+        if(i_iter==100)   LAST_ITERATION = 1;
 		//Rprintf("\t\t Last_iteration value: %d\n",LAST_ITERATION);
     }
 	
@@ -683,6 +755,11 @@ if(verbose >3) Rprintf("check point 3: before loop \n");
 	Free(Unused);
 	Free(IniLogic);
 	Free(BASIS_Targets);
+	//Free(BASIS_PHI);	
+	for(i=0;i<M;i++)
+	{
+		Free(BASIS_PHI[i]);
+	}
 	Free(BASIS_PHI);
 	Free(S_in);
 	Free(Q_in);
@@ -699,7 +776,8 @@ if(verbose >3) Rprintf("check point 3: before loop \n");
 	Free(SIGMANEW);
 	Free(PHIsig);
 	
-	
+		//Block Coordinate Ascent Algorithm Parameters;
+	Free(nuAction);
 }
 
 /****************************************************************************/
@@ -710,7 +788,7 @@ void fEBInitializationGfNeEN(double *Alpha, double *PHI, int *Used, int *Unused,
 			double *Targets, double *Scales, int * initial, int N, int *m, int K, double *beta)
 {
     //basis dimension
-    int M,M_full,i,j,kk,k,index;
+    int M,M_full,i,j,kk,index;//k,
   	kk					= K;
     M_full				= K*(K+1)/2;
 	//M_full					= K;
@@ -749,14 +827,14 @@ void fEBInitializationGfNeEN(double *Alpha, double *PHI, int *Used, int *Unused,
             readPtr1 = &BASIS[i*N];
 			proj = F77_CALL(ddot)(&N, readPtr1, &inci,Targets, &incj);
 			proj			= proj/Scales[i];
-            if(fabs(proj) > fabs(proj_ini))
+            if(fabs(proj) < fabs(proj_ini))
             {
                 proj_ini    = proj;
                 loc1        = i;
                 loc2        = i;
                 Used[0]		= i + 1;
             }
-        }
+        }/*
         for(i=0;i< (K - 1);i++)
         {
             for(j= (i+ 1); j<K; j++)
@@ -774,6 +852,7 @@ void fEBInitializationGfNeEN(double *Alpha, double *PHI, int *Used, int *Unused,
                 kk              = kk + 1;
             }
         }
+		*/
         //PHI2, duplicate for linear solver
         
         //
@@ -873,7 +952,7 @@ void LinearSolverGfNeEN(double * a, double *logout, int N, int M,double *output)
  /// ***********************************************************************************************   
 
 
-void CacheBPGfNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
+void CacheBPGfNeEN(double **BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
 				double *Targets, double *scales,int N,int K,int M,int M_full)
 {
 	double	zTargets;
@@ -895,7 +974,8 @@ void CacheBPGfNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, doub
 				cache2[h*M+l]		= PHI[l*N+h] * BASIS[i*N+ h];
 				z2[l]				= z2[l] + cache2[h*M+l];
 			}
-			BASIS_PHI[l*M_full+i]	= z2[l]/scales[i];
+			//BASIS_PHI[l*M_full+i]	= z2[l]/scales[i];
+			BASIS_PHI[l][i]	= (z2[l]/scales[i]);
 		}
 
 		zTargets					= 0;
@@ -915,7 +995,8 @@ void CacheBPGfNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, doub
 				{
 					z2[h]	= 0;
 					for(l =0;l<N;l++) z2[h] = z2[h] + cache2[l*M+h]*BASIS[j*N+l];
-					BASIS_PHI[h*M_full+kk]	= z2[h]/scales[kk];
+					//BASIS_PHI[h*M_full+kk]	= z2[h]/scales[kk];
+					BASIS_PHI[h][kk]	= z2[h]/scales[kk];
 				}
 
 				zTargets			= 0;
@@ -937,12 +1018,12 @@ void CacheBPGfNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, doub
     //Targets: nx1
 
 void fEBLinearFullStatGfNeEN(double *beta, double * SIGMA, double *H, double *S_in, double * Q_in, double * S_out, 
-				double * Q_out,   double *BASIS, double * Scales, double *PHI, double *BASIS_PHI,
+				double * Q_out,   double *BASIS, double * Scales, double *PHI, double **BASIS_PHI,
 				double *BASIS_Targets, double * Targets, int * Used, double *Alpha, double * Mu, 
 				 double *gamma,int *n, int *m, int* kdim, int *iteration,int *i_iter)
 {
     //basis dimension
-    int N,K,M,i;
+    int N,K,M,i,j,p;
    	N					= *n;			// row number
     K					= *kdim;		// column number
 	int M_full;
@@ -951,7 +1032,7 @@ void fEBLinearFullStatGfNeEN(double *beta, double * SIGMA, double *H, double *S_
 	//lapack
 	int inci =1;
 	int incj =1;
-	double *readPtr1;
+	//double *readPtr1;
 	double a_blas = 1;
 	double b_blas = 1;
 
@@ -1019,25 +1100,29 @@ void fEBLinearFullStatGfNeEN(double *beta, double * SIGMA, double *H, double *S_
 
     for(i=0; i<M_full; i++)
     {
-		//for(j=0;j<M;j++)
-		//{
-		//	BPvector[j]			= 0;
+		for(j=0;j<M;j++)
+		{
+			BPvector[j]			= 0;
 		//	for(p=0;p<M;p++)	BPvector[j]		= BPvector[j] + BASIS_PHI[p*M_full + i]*SIGMA[j*M+p]; 
-		//}
-		readPtr1 	= &BASIS_PHI[i];
-		b_blas 		= 0;
-		F77_CALL(dgemv)(&transa, &M, &M,&a_blas, SIGMA, &M, readPtr1, &M_full, &b_blas,BPvector, &incj);
+			for(p=0;p<M;p++)	BPvector[j]		= BPvector[j] + BASIS_PHI[p][i]*SIGMA[j*M+p]; 
+		}
+		//readPtr1 	= &BASIS_PHI[i];
+		//b_blas 		= 0;
+		//F77_CALL(dgemv)(&transa, &M, &M,&a_blas, SIGMA, &M, readPtr1, &M_full, &b_blas,BPvector, &incj);
 		
 		
         tempSum					= 0;
 		//for(j=0;j<M;j++)		tempSum			= tempSum + BPvector[j]*BASIS_PHI[j*M_full+i];
-		readPtr1 	= &BASIS_PHI[i];
-		tempSum		= F77_CALL(ddot)(&M, BPvector, &inci,readPtr1, &M_full);
+		for(j=0;j<M;j++)		tempSum			= tempSum + BPvector[j]*BASIS_PHI[j][i];
+		//readPtr1 	= &BASIS_PHI[i];
+		//tempSum		= F77_CALL(ddot)(&M, BPvector, &inci,readPtr1, &M_full);
+		
 		S_in[i]					= beta[0] - beta[0]*tempSum*beta[0];
 		tempBPMu				= 0;
 		//for(p=0;p<M;p++) tempBPMu = tempBPMu + BASIS_PHI[p*M_full + i] *Mu[p];	
-		readPtr1 	= &BASIS_PHI[i];
-		tempBPMu 	= F77_CALL(ddot)(&M, Mu, &inci,readPtr1, &M_full);
+		for(p=0;p<M;p++) tempBPMu = tempBPMu + BASIS_PHI[p][i] *Mu[p];
+		//readPtr1 	= &BASIS_PHI[i];
+		//tempBPMu 	= F77_CALL(ddot)(&M, Mu, &inci,readPtr1, &M_full);
 		
 		Q_in[i]					= beta[0]*(BASIS_Targets[i] - tempBPMu);
         //S_out[i]         		= S_in[i];
@@ -1098,7 +1183,8 @@ void MatrixInverseGfNeEN(double * a,int N)
 //***************************************************************************************************
 void fEBDeltaMLGfNeEN(double *DeltaML, int *Action, double *AlphaRoot, int *anyToDelete,
 				int *Used, int * Unused, double * S_out, double * Q_out, double *Alpha,
-				double *a_lambda, double *b_Alpha, int N_used, int N_unused)
+				double *a_lambda, double *b_Alpha, int N_used, int N_unused,
+				double *deltaLogMarginal, int *nu)
 {
     //basis dimension
     int M_full,i,index;
@@ -1115,10 +1201,16 @@ anyToDelete[0] = 0;
 	lambda1 							= *a_lambda * (b_Alpha[0]);
 	lambda2 							= *a_lambda * (1 - b_Alpha[0]);
     //int   anyToDelete					= 0;
-    int   anyToAdd						= 0;
-    const int CNPriorityAddition		= 0;
-    const int CNPriorityDeletion		= 1;
+    //int   anyToAdd						= 0;
+    //const int CNPriorityAddition		= 0;
+    //const int CNPriorityDeletion		= 1;
 	//Rprintf("Inside fEBDeltaMLBmNeEN: a: %f, b %f \n",a, b);
+	
+	//Block Coordinate Ascent Algorithm Parameters;
+	for(i=0;i<M_full;i++) Action[i] = -10;
+	double deltaMLmax = 0;
+	int 	max = 0;
+	
     for(i=0;i<N_used;i++)
     {
         //anyToDelete				= false;
@@ -1157,6 +1249,14 @@ anyToDelete[0] = 0;
             logL                = (log(oldAlpha/(oldAlpha+S_out[index] + lambda2)) + pow(Q_out[index],2)/(oldAlpha + S_out[index] + lambda2))*0.5 - lambda1/oldAlpha;
             DeltaML[index]      = - logL;
         }
+		
+		//Block Coordinate Ascent Algorithm Parameters;
+		if(DeltaML[index]>deltaMLmax)
+		{
+				max			= index;
+				deltaMLmax 	= DeltaML[index];
+		}
+		
     }
     //ADDITION
     for(i=0;i<N_unused;i++)
@@ -1181,10 +1281,44 @@ anyToDelete[0] = 0;
             }
         }
         //case 2
-        //case 3        
+        //case 3  
+		//Block Coordinate Ascent Algorithm Parameters;
+		if(DeltaML[index]>deltaMLmax)
+		{
+				max			= index;
+				deltaMLmax 	= DeltaML[index];
+		} 		
     }
-        
-	//
+    
+	//Block Coordinate Ascent Algorithm Parameters; 
+	// only one action for a block
+	int selectedAction = Action[max];
+	
+	/*
+	if(i_iter <=(1*N_used)) // re-estimate
+	{
+		
+		for(i=0;i<M_full;i++)
+		{
+			if(Action[i] !=ACTION_REESTIMATE) DeltaML[i] = 0;
+		}
+	}else
+	{
+		for(i=0;i<M_full;i++)
+		{
+			if(Action[i] !=selectedAction) DeltaML[i] = 0;
+		}
+	}
+	*/
+		for(i=0;i<M_full;i++)
+		{
+			if(Action[i] !=selectedAction) DeltaML[i] = 0;
+		}
+	
+deltaLogMarginal[0] = deltaMLmax;
+nu[0] 	= max;
+    
+	/*
     if((anyToAdd==1 && CNPriorityAddition==1) || (anyToDelete[0]==1 && CNPriorityDeletion==1))
     {
         for(i=0;i<M_full;i++)
@@ -1198,11 +1332,13 @@ anyToDelete[0] = 0;
                     if(anyToDelete[0] ==1 && CNPriorityDeletion==1 && CNPriorityAddition!=1) DeltaML[i] = 0;
             }
         }
-    }    
+    }  
+
+	*/
 }
 
 //888888888888888888888888888888888888888888888888888888888888888888888888888888888888
-int ActionAddGfNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
+int ActionAddGfNeEN(double **BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
 			double *beta, double* Alpha, double newAlpha, double*SIGMA, double*Mu, double*S_in,
 			double*Q_in, int nu, double*SIGMANEW, int M_full,int N, int K, int M)
 {
@@ -1322,16 +1458,17 @@ int ActionAddGfNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI,
 	{
 		temp				= 0;
 		//for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j*M_full + i]*tmpp[j];
-		readPtr1 			= &BASIS_PHI[i];
-		temp 				= F77_CALL(ddot)(&M, readPtr1, &M_full,tmpp,&incj); 
+		for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j][i]*tmpp[j];
+		//readPtr1 			= &BASIS_PHI[i];
+		//temp 				= F77_CALL(ddot)(&M, readPtr1, &M_full,tmpp,&incj); 
 		mCi[i]				= BASIS_B_Phi[i] -  beta[0]*temp;
-		BASIS_PHI[M*M_full + i]					= BASIS_Phi[i];
+		//BASIS_PHI[M*M_full + i]					= BASIS_Phi[i];
 		S_in[i]				= S_in[i] - mCi[i]*mCi[i]*s_ii;
 		Q_in[i]				= Q_in[i] - mu_i*mCi[i];
 	}
-	
+	BASIS_PHI[M]=BASIS_Phi;
 	int UPDATE_REQUIRED		= 1;
-	Free(BASIS_Phi);
+	//Free(BASIS_Phi);
 	Free(BASIS_B_Phi);
 	Free(mCi);
 	Free(z);
@@ -1346,7 +1483,7 @@ int ActionAddGfNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI,
 
  
  
- int ActionDelGfNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double*BASIS_PHI,
+ int ActionDelGfNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double**BASIS_PHI,
 		double*Mu, double*S_in, double*Q_in, double *beta, int jj, int N, int M, int M_full)
  {
 	 int i,j;
@@ -1384,7 +1521,7 @@ int ActionAddGfNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI,
 	{
 		for(j=0;j<index;j++) SIGMANEW[j*index + i]	= tempSIGMA[j*M + i];
 	}
-	if(jj != index)
+	if(jj != index)// incase the one to be deleted is the last column.
 	{
 	//step 1: last column
 	readPtr1 = &SIGMANEW[jj*index];
@@ -1424,17 +1561,21 @@ int ActionAddGfNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI,
 	for(i=0;i<M_full;i++)
 	{
 		temp				= 0;
-		for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
+		//for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
+		for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j][i]*SIGMA[jj*M + j];
 		S_in[i]				= S_in[i] +  pow(beta[0]*temp,2)/SIGMA[jj*M + jj];
 		Q_in[i]				= Q_in[i] +  beta[0]*temp *Mujj /SIGMA[jj*M + jj];
 		//jPm = beta*temp
 	}
-	
+	double *ptr;
+	ptr=BASIS_PHI[jj];
+	BASIS_PHI[jj]=BASIS_PHI[index];
+	BASIS_PHI[index]=ptr;
 	//BASIS_PHI(:,jj) = [];
 	//for(i=0;i<M_full;i++) BASIS_PHI[jj*M_full + i]	= BASIS_PHI[index*M_full+i];
-	readPtr1 = &BASIS_PHI[jj*M_full];
-	readPtr2 = &BASIS_PHI[index*M_full];
-	F77_CALL(dcopy)(&M_full,readPtr2,&inci,readPtr1,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+	//readPtr1 = &BASIS_PHI[jj*M_full];
+	//readPtr2 = &BASIS_PHI[index*M_full];
+	//F77_CALL(dcopy)(&M_full,readPtr2,&inci,readPtr1,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
 	
 	int UPDATE_REQUIRED		=1;
 	Free(tempSIGMA);
