@@ -12,32 +12,36 @@ void LinearSolverGmNeEN(double * a, double *logout, int N,int M,double *output);
 void fEBInitializationGmNeEN(double *Alpha, double * PHI, int *Used, int *Unused, 
 				double *BASIS, double *Targets, double *Scales, int * initial, 
 				int n, int *m, int kdim, double *beta);
-
+				
+void fEBInitializationGmNeEN2(double *Alpha, double *PHI, int *Used, int *Unused, double *BASIS, 
+			double *Targets, double *Scales, int * initial, int N, int *m, int K, double *beta);
+			
 void MatrixInverseGmNeEN(double * a,int N);
-void CacheBPGmNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
+void CacheBPGmNeEN(double **BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
 				double *Targets,double *scales,int N,int K,int M, int M_full);
 
 void fEBLinearFullStatGmNeEN(double *beta,double * SIGMA,double *H, double *S_in, double * Q_in, double * S_out, 
-				double * Q_out,  double *BASIS, double * Scales, double *PHI, double *BASIS_PHI,
+				double * Q_out,  double *BASIS, double * Scales, double *PHI, double **BASIS_PHI,
 				double *BASIS_Targets, double * Targets, int * Used, double *Alpha, double * Mu, 
 				 double *Gamma,int *n, int *m, int* kdim, int *iteration,int *i_iter);
 
 void fEBDeltaMLGmNeEN(double *DeltaML, int *Action, double *AlphaRoot, int *anyToDelete,
 				int *Used, int * Unused, double * S_out, double * Q_out, double *Alpha,
 				double *a_lmabda, double *b_Alpha,
-				int m, int mBar);
+				int m, int mBar,
+				double *deltaLogMarginal,int *nu);
 
 void LinearFastEmpBayesGmNeEN(int *Used, double *Mu, double *SIGMA, double *H, double *Alpha, double *PHI,
 				double *BASIS, double * Targets, double *Scales, double *a_lambda,double *b_Alpha,
 				int *iteration, int *n, int *kdim, int *m, int basisMax, double *b, double *beta,double * C_inv,
 				int verbose);
 
-int ActionAddGmNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
+int ActionAddGmNeEN(double **BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
 			double *beta, double* Alpha, double newAlpha, double*SIGMA, double*Mu, double*S_in,
 			double*Q_in, int nu, double*SIGMANEW, int M_full, int N, int K, int M);
 
 
-int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double*BASIS_PHI,
+int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double**BASIS_PHI,
 				double*Mu, double*S_in, double*Q_in, double *beta, int jj, int N, int M, int M_full);
 
 double varTargetsGmNeEN(double* Target,int N);
@@ -227,6 +231,7 @@ void elasticNetLinearNeMainEff(double *BASIS, double *y, double *a_lambda, doubl
 	Free(m);
 	Free(C_inv);	
 	Free(tempW);
+	Free(Csum);
 }
 
 
@@ -258,6 +263,12 @@ void LinearFastEmpBayesGmNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	const int		CNBetaUpdateStart	=10;
 	const double	BetaMaxFactor		=1e6;
 	const double	MinDeltaLogBeta		=1e-6;
+	
+	//Block Coordinate Ascent Algorithm Parameters;
+	const double 	nAdd 				= 0.9; //only deltaML >nAdd will be considered as candidate
+	double nAdd_real 					= nAdd;
+	const double 	MLdelta 			= 0.001; //only deltaML > MLdelta will be selected for action.
+	
     //[Alpha,PHI2,Used,Unused,Mu2]=InitialCategory(BASIS,Targets,Scales,PHI2,Used,Alpha,Mu2,IniLogic) 
     int *IniLogic;
 	IniLogic				= (int*) Calloc(1,int);
@@ -278,7 +289,7 @@ void LinearFastEmpBayesGmNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	//lapack
 	int inci 				=1;
 	int incj 				=1;
-	double *readPtr1, *readPtr2;
+	double *readPtr1;//, *readPtr2;
 	int inc0 				= 0;
 	double a_blas 			= 1;
 	double b_blas 			= 1;
@@ -292,15 +303,22 @@ void LinearFastEmpBayesGmNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	//Rprintf("N_used is: %d; N_unused:%d, M: %d,sample size: %d \n",N_used,N_unused,M,N);
 
 	fEBInitializationGmNeEN(Alpha, PHI, Used, Unused, BASIS, Targets, Scales, IniLogic, N, m, K,beta);
+	M = m[0];
+	N_unused = M_full - M;
+	
 	//Rprintf("\t Initialized basis %d, Alpha: %f, \n", Used[0],Alpha[0]);
 	//Rprintf("\t beta: %f\n",beta[0]);
 	//Rprintf("\t m: %d\n",M);
 	//for(i=0;i<N;i++) Rprintf("\tPHI: \t %f\n",PHI[i]);
 	//for(i=0;i<10;i++) Rprintf("PHI2: %f \t  %f; BASIS: %f\n",PHI2[i],PHI2[N+i],BASIS[181*N+i]/Scales[181]);
     //CACHE MATRIX
-	double *BASIS_Targets,*BASIS_PHI;
+	double *BASIS_Targets,**BASIS_PHI;
 	BASIS_Targets		= (double *) Calloc(M_full,double);
-	BASIS_PHI			= (double *) Calloc(M_full*basisMax,double);
+	BASIS_PHI			= (double **) Calloc(basisMax,double);
+	for(i=0;i<M;i++)
+	{
+		BASIS_PHI[i] 	= (double *) Calloc(M_full,double);
+	}
 	CacheBPGmNeEN(BASIS_PHI, BASIS_Targets, BASIS, PHI,	Targets,Scales,N,K,M,M_full);
 	
 	double *S_in, *Q_in, *S_out, *Q_out,*gamma;
@@ -322,7 +340,8 @@ void LinearFastEmpBayesGmNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
    double *DeltaML, *AlphaRoot,deltaLogMarginal,*phi,newAlpha,oldAlpha;
     double deltaInv,kappa,Mujj;
     //
-	int *Action, *anyToDelete,selectedAction;
+	int *Action, *anyToDelete;
+	int selectedAction = -10;
 	anyToDelete			= (int*) Calloc(1,int);
 	DeltaML				=	(double *) Calloc(M_full,double);
 	AlphaRoot			=	(double *) Calloc(M_full,double);
@@ -348,220 +367,279 @@ void LinearFastEmpBayesGmNeEN(int *Used, double *Mu, double *SIGMA, double *H, d
 	
 	double temp;			// for action_reestimate
 	double * SIGMANEW	= (double * ) Calloc(basisMax*basisMax, double);
-if(verbose>3) Rprintf("check point 3: before loop \n");
+
+//Block Coordinate Ascent Algorithm Parameters;
+	double cutoff;
+	int *nuAction;
+	nuAction = (int*) Calloc(M_full,int);
+	int nUpdate, iU;// nTerminate;
+	
+	if(verbose>3 && iter ==1)
+	{
+		Rprintf("check point 3: before loop,initial number of basis:%d\t Basis initialized: ",M);
+		for(i=0;i<M;i++) Rprintf("%dth:%d\t",i,Used[i]);
+		Rprintf("\n");
+		
+	}
    while(LAST_ITERATION!=1)
     {
         i_iter						= i_iter + 1;
-		if(verbose >4) Rprintf("\t inner loop %d \n",i_iter);
+		if(verbose >4) Rprintf("\t inner loop %d; number of basis: %d \n",i_iter, M);
         //M							= N_used + 1;
      	//N_unused					= M_full - N_used;
 
         //[DeltaML,Action,AlphaRoot,anyToDelete]     = fEBDeltaMLGmNeEN(Used,Unused,S_out,Q_out,Alpha,a,b);
 		fEBDeltaMLGmNeEN(DeltaML, Action, AlphaRoot,anyToDelete,Used, Unused, S_out, Q_out, Alpha,
-				a_lambda, b_Alpha, M, N_unused);
+				a_lambda, b_Alpha, M, N_unused,
+				&deltaLogMarginal,&nu);
 		//
-        deltaLogMarginal			= 0;
-        nu							= -1;
-        for(i=0;i<M_full;i++)
-        {
-            if(DeltaML[i]>deltaLogMarginal)
-            {
-                deltaLogMarginal    = DeltaML[i];
-                nu                  = i;
-            }
-        }
+        //deltaLogMarginal			= MLdelta;
+        //nu							= -1;
+        //for(i=0;i<M_full;i++)
+        //{
+        //    if(DeltaML[i]>deltaLogMarginal)
+         //   {
+        //        deltaLogMarginal    = DeltaML[i];
+         //       nu                  = i;
+         //   }
+        //}
         //selectedAction          = Action[nu];
 		
-        if(nu==-1)
+		if(nu==-1)
 		{		
+			if(verbose >4) Rprintf("\t No worthwhile action; Terminate inner loop.\n");
 			anyWorthwhileAction     = 0;
-			selectedAction          = -10;
+			selectedAction          = ACTION_TERMINATE;
 		}else
 		{
 			anyWorthwhileAction	= 1;
-			selectedAction          = Action[nu];
-			newAlpha                = AlphaRoot[nu];
-		}
-		//Rprintf("\t ActionOn nu= %d, deltaML: %f, selectedAction: %d, Alpha: %f\n",nu+1, DeltaML[nu],selectedAction,AlphaRoot[nu]);
-        if(selectedAction==ACTION_REESTIMATE || selectedAction==ACTION_DELETE)
-        {
-            index                   = nu + 1; 
-            for(i=0;i<M;i++)
-            {
-                if (Used[i]==index)	
-				{
-						jj  = i;
-						break;
-				}
-            }
-        }
-
-        //kk                          = K;                          
-        for(i=0;i<K;i++)
-        {
-            if (i==nu)
-            {
-                //for(L=0;L<N;L++)    phi[L]  = BASIS[i*N+L]/Scales[i];
-				readPtr1 		= &BASIS[i*N];
-				F77_CALL(dcopy)(&N,readPtr1,&inci,phi,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
-				b_blas 			= 1/Scales[i];
-				F77_CALL(dscal)(&N,&b_blas,phi,&inci); 		//dscal(n, a, x, incx) x = a*x
-				//break;
-            }
-			/*else if(i<(K-1))
-            {
-                for(j=(i+1);j<K;j++)
-                {
-                    if(kk==nu)
-                    {
-                        for(L=0;L<N;L++)    phi[L] =BASIS[i*N+L]*BASIS[j*N+L]/Scales[kk];
-						//break;
-                    }
-                    kk              = kk + 1;
-                }
-            }*/
-        }
-		//Rprintf("N: %d, kk: %d,i: %d, j: %d, K: %d\n",N,kk,i,j,K);
-		//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
-        //newAlpha                    = AlphaRoot[nu];
-		//Rprintf("\tNew alpha: %f\n",newAlpha);
-        if(anyWorthwhileAction==0)  selectedAction = ACTION_TERMINATE;
-        if(selectedAction==ACTION_REESTIMATE)
-        {
-            if (fabs(log(newAlpha)-log(Alpha[jj]))<=1e-3 && anyToDelete[0] ==0)
-            {	
-    			//cout<<"Terminated by small deltaAlpha on basis:"<<nu<<endl;
-                selectedAction		= ACTION_TERMINATE;
-            }
-        }
-	//Rprintf("\t selectedAction: %d\n",selectedAction);
-        //
-        UPDATE_REQUIRED				= 0;
-        if(selectedAction==ACTION_REESTIMATE)
-        {
-			if(verbose>4) Rprintf("\t\t Action: Reestimate : %d \t deltaML: %f\n",nu + 1, deltaLogMarginal);
-            oldAlpha				= Alpha[jj];
-            Alpha[jj]				= newAlpha;
-
-            deltaInv				= 1.0/(newAlpha-oldAlpha);
-            kappa					= 1.0/(SIGMA[jj*M+jj] + deltaInv);
-            Mujj					= Mu[jj];
-            //for(i=0;i<M;i++)		Mu[i]    = Mu[i] - Mujj *kappa * SIGMA[jj*M+i];
-			readPtr1 				= &SIGMA[jj*M];
-			b_blas 					= -Mujj * kappa;
-			F77_CALL(daxpy)(&M, &b_blas,readPtr1, &inci,Mu, &incj); //daxpy(n, a, x, incx, y, incy) y := a*x + y
-
- 			//	SIGMANEW	=SIGMA-sjj;
-			for(i=0;i<M;i++)
+			//Block Coordinate Ascent Algorithm Parameters;
+			// find a block to update;
+			if(Action[nu] == ACTION_ADD)
 			{
-				for(j=0;j<M;j++)	SIGMANEW[j*M + i] = SIGMA[j*M + i] - kappa * SIGMA[jj*M+i]*SIGMA[jj*M+j];
+				nAdd_real			= nAdd;				
+			}else
+			{
+				nAdd_real 			= 1;
+
 			}
-			
-			//S_in		= S_in + kappa*(beta*BASIS_PHI * s_j).^2;
-			//Q_in		= Q_in - beta*BASIS_PHI*(deltaMu);           
-			
+			cutoff 					= deltaLogMarginal * nAdd_real;
+			if(cutoff< MLdelta)
+			{
+				cutoff 				= MLdelta;
+			}
+			nUpdate 				= 0;
 			for(i=0;i<M_full;i++)
 			{
-				temp	= 0;
-				//for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
-				readPtr1 			= &BASIS_PHI[i];
-				readPtr2 			= &SIGMA[jj*M];
-				temp = F77_CALL(ddot)(&M, readPtr1, &M_full,readPtr2, &inci);
-				S_in[i]				= S_in[i] +  pow(beta[0]*temp,2)*kappa;
-				Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
+
+				if(DeltaML[i]>=cutoff)
+				{
+					nuAction[nUpdate]= i;
+					nUpdate++;
+				}
+				
 			}
-			//
-			//for(i=0;i<M_full;i++)
-			//{
-			//	temp	= 0;
-			//	for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]* SIGMA[jj*M+j];
-			//	Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
-			//}
-			UPDATE_REQUIRED			= 1;
-        }
-        /////////////////////////////////////////////////////////////////////////////////
-        else if(selectedAction==ACTION_ADD)
-        {
-			if(verbose>4) Rprintf("\t\t Action:add : %d \t deltaML: %f\n",nu + 1,deltaLogMarginal);
-			//Rprintf("\t\t newAlpha: %f\n",newAlpha);
-            // B_phi*PHI2*SIGMA2        tmp = B_phi*PHI2 
-            index					= M + 1;
-			if(index > (basisMax -10) && iter>1) {
-				Rprintf("bases: %d, warning: out of Memory!\n",index);
-			}//return;
-			if(index > (basisMax -1) && iter>1) {
-				Rprintf("bases: %d, out of Memory,exiting program!\n",index);
-				//exit(1);			
-			}
-			//BASIS_B_Phi	=BASIS_Phi*beta;
-			UPDATE_REQUIRED		= ActionAddGmNeEN(BASIS_PHI, BASIS, Scales, PHI, phi, beta, Alpha,
-				newAlpha, SIGMA, Mu, S_in, Q_in, nu, SIGMANEW, M_full, N, K, M);
-
-            //			
-            Used[M]			= nu + 1;						//new element
-
-            //
-
-			N_unused				= N_unused - 1;
-			for(i=0;i<N_unused;i++)
-            {                
-                if(Unused[i]== (nu + 1))		Unused[i] =Unused[N_unused];
-            }
-			m[0]					= M + 1;
-			M						= m[0];
-			//for(i=0;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i],Mu[i]);
-			//Rprintf(" \t\t S_in[73]: %f, Q_in[73]: %f \n",S_in[nu],Q_in[nu]);
-			//for(i=0;i<4;i++) Rprintf(" \t\t SIGMANEW: %f \n",SIGMANEW[i]);
-			//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
-        }
-		//
-        else if(selectedAction==ACTION_DELETE)
-        {
-			if(verbose>4) Rprintf("\t\t Action: delete : %d deltaML: %f \n",nu + 1,deltaLogMarginal);
-            UPDATE_REQUIRED = ActionDelGmNeEN(PHI, Alpha, SIGMA, SIGMANEW, BASIS_PHI,
-				Mu, S_in, Q_in, beta, jj, N, M, M_full);
-			index					= M -1;
-            //Used; Unused;
-            Used[jj]				= Used[index];
-
-            //
-			N_unused				= N_unused + 1;
-			Unused[N_unused -1]		= nu + 1;
-
-			m[0]					= M -1;
-			M						= m[0];
-        //for(i=1;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i-1],Mu2[i]);
-		}
-
-        //Rprintf("\t\t Update_required: %d\n",UPDATE_REQUIRED);
-        //
-		if(UPDATE_REQUIRED==1)
-        {
-			//for(i=0;i<M_full;i++)
-			//{
-			//	S_out[i]	= S_in[i];
-			//	Q_out[i]	= Q_in[i];
-			//}
-			F77_CALL(dcopy)(&M_full,S_in,&inci,S_out,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
-			F77_CALL(dcopy)(&M_full,Q_in,&inci,Q_out,&incj); 
-			for(i=0;i<M;i++)
-		    {
-				index					= Used[i] -1;
-				S_out[index]			= Alpha[i]*S_in[index]/(Alpha[i]-S_in[index]);
-				Q_out[index]			= Alpha[i]*Q_in[index]/(Alpha[i]-S_in[index]);
-			}
-			//for(i=0;i<M;i++)
-			//{
-			//	for(j=0;j<M;j++) SIGMA[i*M+j]	= SIGMANEW[i*M+j];
-			//}
-			MM = M*M;			
-			F77_CALL(dcopy)(&MM,SIGMANEW,&inci,SIGMA,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
-
+			if(Action[nu] == ACTION_DELETE && nUpdate>1) nUpdate = 1; // when deleting co-linear basis: delete one at a time
+			//if(verbose >4) Rprintf("\t deltaML cutoff: %f ; number to be updated: %d \n",cutoff, nUpdate);
 			
-			for(i=0;i<M;i++) gamma[i]		= 1- Alpha[i]*SIGMA[i*M+i];
-        }
 
-		if ((selectedAction==ACTION_TERMINATE)||(i_iter<=CNBetaUpdateStart)||(i_iter%5==0))
+				
+			for(iU=0;iU<nUpdate;iU++) //update each basis
+			{
+				nu 					= nuAction[iU];
+								
+				selectedAction          = Action[nu];
+				newAlpha                = AlphaRoot[nu];
+				deltaLogMarginal 	= DeltaML[nu];
+
+				//Rprintf("\t ActionOn nu= %d, deltaML: %f, selectedAction: %d, Alpha: %f\n",nu+1, DeltaML[nu],selectedAction,AlphaRoot[nu]);
+				if(selectedAction==ACTION_REESTIMATE || selectedAction==ACTION_DELETE)
+				{
+					index                   = nu + 1; 
+					for(i=0;i<M;i++)
+					{
+						if (Used[i]==index)	
+						{
+								jj  = i;
+								break;
+						}
+					}
+				}
+
+				//kk                          = K;                          
+				for(i=0;i<K;i++)
+				{
+					if (i==nu)
+					{
+						//for(L=0;L<N;L++)    phi[L]  = BASIS[i*N+L]/Scales[i];
+						readPtr1 		= &BASIS[i*N];
+						F77_CALL(dcopy)(&N,readPtr1,&inci,phi,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+						b_blas 			= 1/Scales[i];
+						F77_CALL(dscal)(&N,&b_blas,phi,&inci); 		//dscal(n, a, x, incx) x = a*x
+						//break;
+					}
+					/*else if(i<(K-1))
+					{
+						for(j=(i+1);j<K;j++)
+						{
+							if(kk==nu)
+							{
+								for(L=0;L<N;L++)    phi[L] =BASIS[i*N+L]*BASIS[j*N+L]/Scales[kk];
+								//break;
+							}
+							kk              = kk + 1;
+						}
+					}*/
+				}
+				//Rprintf("N: %d, kk: %d,i: %d, j: %d, K: %d\n",N,kk,i,j,K);
+				//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
+				//newAlpha                    = AlphaRoot[nu];
+				//Rprintf("\tNew alpha: %f\n",newAlpha);
+				if(anyWorthwhileAction==0)  selectedAction = ACTION_TERMINATE;
+				if(selectedAction==ACTION_REESTIMATE)
+				{
+					if (fabs(log(newAlpha)-log(Alpha[jj]))<=1e-3 && anyToDelete[0] ==0)
+					{	
+						//cout<<"Terminated by small deltaAlpha on basis:"<<nu<<endl;
+						selectedAction		= ACTION_TERMINATE;
+					}
+				}
+				//Rprintf("\t selectedAction: %d\n",selectedAction);
+				//
+				UPDATE_REQUIRED				= 0;
+				if(selectedAction==ACTION_REESTIMATE)
+				{
+					if(verbose>4) Rprintf("\t\t Action: Reestimate : %d \t deltaML: %f Alpha: %f s_i: %f q_i: %f S_i: %f Q_i: %f\n",
+								nu + 1, deltaLogMarginal,newAlpha,S_out[nu],Q_out[nu],S_in[nu],Q_in[nu]);
+					oldAlpha				= Alpha[jj];
+					Alpha[jj]				= newAlpha;
+
+					deltaInv				= 1.0/(newAlpha-oldAlpha);
+					kappa					= 1.0/(SIGMA[jj*M+jj] + deltaInv);
+					Mujj					= Mu[jj];
+					//for(i=0;i<M;i++)		Mu[i]    = Mu[i] - Mujj *kappa * SIGMA[jj*M+i];
+					readPtr1 				= &SIGMA[jj*M];
+					b_blas 					= -Mujj * kappa;
+					F77_CALL(daxpy)(&M, &b_blas,readPtr1, &inci,Mu, &incj); //daxpy(n, a, x, incx, y, incy) y := a*x + y
+
+					//	SIGMANEW	=SIGMA-sjj;
+					for(i=0;i<M;i++)
+					{
+						for(j=0;j<M;j++)	SIGMANEW[j*M + i] = SIGMA[j*M + i] - kappa * SIGMA[jj*M+i]*SIGMA[jj*M+j];
+					}
+					
+					//S_in		= S_in + kappa*(beta*BASIS_PHI * s_j).^2;
+					//Q_in		= Q_in - beta*BASIS_PHI*(deltaMu);           
+					
+					for(i=0;i<M_full;i++)
+					{
+						temp	= 0;
+						//for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
+						//readPtr1 			= &BASIS_PHI[i];
+						//readPtr2 			= &SIGMA[jj*M];
+						//temp = F77_CALL(ddot)(&M, readPtr1, &M_full,readPtr2, &inci);
+						for(j=0;j<M;j++) temp = temp + BASIS_PHI[j][i]*SIGMA[jj*M + j];
+						S_in[i]				= S_in[i] +  pow(beta[0]*temp,2)*kappa;
+						Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
+					}
+					//
+					//for(i=0;i<M_full;i++)
+					//{
+					//	temp	= 0;
+					//	for(j=0;j<M;j++) temp = temp + BASIS_PHI[j*M_full + i]* SIGMA[jj*M+j];
+					//	Q_in[i]				= Q_in[i] +  beta[0]*Mujj *kappa*temp;
+					//}
+					UPDATE_REQUIRED			= 1;
+				}
+				/////////////////////////////////////////////////////////////////////////////////
+				else if(selectedAction==ACTION_ADD)
+				{
+					if(verbose>4) Rprintf("\t\t Action:add : %d \t deltaML: %f Alpha: %f s_i: %f q_i: %f S_i: %f Q_i: %f\n",
+								nu + 1, deltaLogMarginal,newAlpha,S_out[nu],Q_out[nu],S_in[nu],Q_in[nu]);
+					//Rprintf("\t\t newAlpha: %f\n",newAlpha);
+					// B_phi*PHI2*SIGMA2        tmp = B_phi*PHI2 
+					index					= M + 1;
+					if(index > (basisMax -10) && iter>1) {
+						Rprintf("bases: %d, warning: out of Memory!\n",index);
+					}//return;
+					if(index > (basisMax -1) && iter>1) {
+						Rprintf("bases: %d, out of Memory,exiting program!\n",index);
+						//exit(1);			
+					}
+					//BASIS_B_Phi	=BASIS_Phi*beta;
+					UPDATE_REQUIRED		= ActionAddGmNeEN(BASIS_PHI, BASIS, Scales, PHI, phi, beta, Alpha,
+						newAlpha, SIGMA, Mu, S_in, Q_in, nu, SIGMANEW, M_full, N, K, M);
+
+					//			
+					Used[M]			= nu + 1;						//new element
+
+					//
+
+					N_unused				= N_unused - 1;
+					for(i=0;i<N_unused;i++)
+					{                
+						if(Unused[i]== (nu + 1))		Unused[i] =Unused[N_unused];
+					}
+					m[0]					= M + 1;
+					M						= m[0];
+					//for(i=0;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i],Mu[i]);
+					//Rprintf(" \t\t S_in[73]: %f, Q_in[73]: %f \n",S_in[nu],Q_in[nu]);
+					//for(i=0;i<4;i++) Rprintf(" \t\t SIGMANEW: %f \n",SIGMANEW[i]);
+					//for(i=0;i<N;i++) Rprintf("phi: %f\n",phi[i]);
+				}
+				//
+				else if(selectedAction==ACTION_DELETE)
+				{
+					if(verbose>4) Rprintf("\t\t Action: delete : %d deltaML: %f Alpha: %f s_i: %f q_i: %f S_i: %f Q_i: %f\n",
+								nu + 1, deltaLogMarginal,newAlpha,S_out[nu],Q_out[nu],S_in[nu],Q_in[nu]);
+					UPDATE_REQUIRED = ActionDelGmNeEN(PHI, Alpha, SIGMA, SIGMANEW, BASIS_PHI,
+						Mu, S_in, Q_in, beta, jj, N, M, M_full);
+					index					= M -1;
+					//free deleted row of BASIS_PHI
+					Free(BASIS_PHI[index]);
+					//Used; Unused;
+					Used[jj]				= Used[index];
+
+					//
+					N_unused				= N_unused + 1;
+					Unused[N_unused -1]		= nu + 1;
+
+					m[0]					= M -1;
+					M						= m[0];
+				//for(i=1;i<M;i++) Rprintf(" \t\t basis: %d :new weight: %f \n",Used[i-1],Mu2[i]);
+				}
+
+				//Rprintf("\t\t Update_required: %d\n",UPDATE_REQUIRED);
+				//
+				if(UPDATE_REQUIRED==1)
+				{
+					//for(i=0;i<M_full;i++)
+					//{
+					//	S_out[i]	= S_in[i];
+					//	Q_out[i]	= Q_in[i];
+					//}
+					F77_CALL(dcopy)(&M_full,S_in,&inci,S_out,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+					F77_CALL(dcopy)(&M_full,Q_in,&inci,Q_out,&incj); 
+					for(i=0;i<M;i++)
+					{
+						index					= Used[i] -1;
+						S_out[index]			= Alpha[i]*S_in[index]/(Alpha[i]-S_in[index]);
+						Q_out[index]			= Alpha[i]*Q_in[index]/(Alpha[i]-S_in[index]);
+					}
+					//for(i=0;i<M;i++)
+					//{
+					//	for(j=0;j<M;j++) SIGMA[i*M+j]	= SIGMANEW[i*M+j];
+					//}
+					MM = M*M;			
+					F77_CALL(dcopy)(&MM,SIGMANEW,&inci,SIGMA,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+
+					
+					for(i=0;i<M;i++) gamma[i]		= 1- Alpha[i]*SIGMA[i*M+i];
+				}
+			} // //end of for(iU)
+		}	
+		
+		if ((selectedAction==ACTION_TERMINATE)||(i_iter<=CNBetaUpdateStart)||(i_iter%5==0) || (nUpdate>=2))
 		{	
 			ee					= 0;
 			//for(i=0;i<N;i++)
@@ -610,7 +688,7 @@ if(verbose>3) Rprintf("check point 3: before loop \n");
 		//Rprintf("\t\t selected Action: %d\n",selectedAction);
 		//
         if(selectedAction==ACTION_TERMINATE) LAST_ITERATION =1;
-        if(i_iter==1000)   LAST_ITERATION = 1;
+        if(i_iter==100)   LAST_ITERATION = 1;
 		//Rprintf("\t\t Last_iteration value: %d\n",LAST_ITERATION);
     }
 	
@@ -657,7 +735,12 @@ if(verbose>3) Rprintf("check point 3: before loop \n");
 	 for(i=0;i<N;i++) C_inv[i*N+i]	=C_inv[i*N+i]	+ beta[0];
 	Free(Unused);	
 	Free(IniLogic);
-	Free(BASIS_Targets);	 
+	Free(BASIS_Targets);
+//Free(BASIS_PHI);	
+	for(i=0;i<M;i++)
+	{
+		Free(BASIS_PHI[i]);
+	}	
 	Free(BASIS_PHI);	
 	Free(S_in);
 	Free(Q_in);
@@ -672,12 +755,185 @@ if(verbose>3) Rprintf("check point 3: before loop \n");
 	Free(e);
 	Free(SIGMANEW);
 	Free(PHIsig);
+	Free(gamma);
+		//
+	//Block Coordinate Ascent Algorithm Parameters;
+	Free(nuAction);
+	
 }
 
 /****************************************************************************/
 
 // [Alpha,PHI2,Used,Unused,Mu2]=InitialCategory(BASIS,Targets,Scales,PHI2,Used,Alpha,Mu2,IniLogic)  //IniLogic: whether input is empty or not
 
+void fEBInitializationGmNeEN2(double *Alpha, double *PHI, int *Used, int *Unused, double *BASIS, 
+			double *Targets, double *Scales, int * initial, int N, int *m, int K, double *beta)
+{
+    //basis dimension
+    int M,M_full,i,j,kk,index;
+  	//kk					= K;
+    //M_full				= K*(K+1)/2;
+	M_full					= K;
+	int IniLogic			= *initial;
+    //INPUT
+    if(IniLogic==0)							// is empty
+    {
+		m[0]				= 1;
+		M					= m[0];
+    }else									// not empty
+    {
+       	M					= m[0];
+    }
+    //output
+    const double init_alpha_max     = 1e3;
+	//lapack
+	int inci =1;
+	int incj =1;
+	double *readPtr1;
+	double b_blas = 1;
+	//lapack end
+    
+	if(IniLogic==0)            // is empty
+    {
+		//Rprintf("\t Inside Initialization, M: %d, K: %d\n",M, K);
+        double proj_ini,proj;
+        int loc1			= 0;
+		int loc2			= 0;
+        proj_ini			= 0;
+		//Used[0]				= 1;
+		
+		//Block Coordinate Ascent Algorithm Parameters;
+		double *projection = (double *) Calloc(M_full,double);
+		double 	nAdd 		= 0.99; //only deltaML >nAdd will be considered as candidate
+		
+        for(i=0;i<K;i++)
+        {
+         	proj			= 0;
+            //for(j=0;j<N;j++)                proj    = proj + BASIS[i*N+j]*Targets[j];
+			readPtr1 = &BASIS[i*N];
+			proj = F77_CALL(ddot)(&N, readPtr1, &inci,Targets, &incj);
+            proj			= proj/Scales[i];
+			projection[i] 	= fabs(proj);
+            if(fabs(proj) > fabs(proj_ini))
+            {
+                proj_ini    = fabs(proj);
+                //loc1        = i;
+                // loc2        = i;
+                //Used[0]		= i + 1;
+            }
+        }
+		//Block Coordinate Ascent Algorithm Parameters;
+		int nIni = 0;
+		double projCut = proj_ini*nAdd;
+		for(i=0;i<K;i++)
+		{
+			if(projection[i]>=projCut) 
+			{
+				Used[nIni] = i+1;
+				nIni++;
+
+			}
+			
+		}
+		M = nIni;
+		m[0]				= M;
+		double *readPtr2;
+		
+		// beta
+		double stdT;
+		stdT					= varTargetsGmNeEN(Targets,N);
+		stdT					= sqrt(stdT);
+		if (stdT<1e-6)	stdT	= 1e-6;
+		beta[0]					= 1/pow((stdT*0.1),2);
+		
+        /*for(i=0;i< (K - 1);i++)
+        {
+            for(j= (i+ 1); j<K; j++)
+            {
+                proj			= 0;
+                for(k=0;k<N;k++)            proj    = proj + BASIS[i*N+k]*BASIS[j*N+k]*Targets[k];
+                proj            = proj/Scales[kk];
+                if(fabs(proj) > fabs(proj_ini))
+                {
+                    proj_ini    = proj;
+                    loc1        = i;
+                    loc2        = j;
+                    Used[0]		= kk  + 1;
+                }
+                kk              = kk + 1;
+            }
+        }*/
+        //PHI2, duplicate for linear solver
+        
+        for(i=0;i<M;i++) //Block Coordinate Ascent Algorithm Parameters;
+		{
+			loc1 = Used[i] -1;
+			loc2 = loc1;
+			//
+			if(loc1==loc2)
+			{
+				//for(i=0;i<N;i++)                PHI[i]			= BASIS[loc1*N+i]/Scales[loc1];
+				readPtr1 		= &BASIS[loc1*N];
+				readPtr2 		= &PHI[i*N];
+				F77_CALL(dcopy)(&N,readPtr1,&inci,readPtr2,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
+				b_blas 			= 1/Scales[loc1];
+				readPtr2 		= &PHI[i*N];
+				F77_CALL(dscal)(&N,&b_blas,readPtr2,&inci); 		//dscal(n, a, x, incx) x = a*x	
+			}else
+			{
+				index				= Used[i] -1;
+				readPtr2 			= &PHI[i*N];
+				for(i=0;i<N;i++)     readPtr2[i]			= BASIS[loc1*N+i]*BASIS[loc2*N+i]/Scales[index];
+			}
+			
+
+		
+
+
+			//Alpha
+			//p                           = diag(PHI'*PHI)*beta;
+			//q                           = (PHI'*Targets)*beta;
+			//Alpha	= p^2/(q^2-p);
+			double p,q;
+			p						= 0;
+			q						= 0;
+			//for(i=0;i<N;i++) 
+			//{
+			//	p					= p + PHI[i]*PHI[i];
+			//	q					= q + PHI[i]*Targets[i];
+			//}
+			readPtr2 = &PHI[i*N];
+			p = F77_CALL(ddot)(&N, readPtr2, &inci,readPtr2, &incj);
+			q = F77_CALL(ddot)(&N, readPtr2, &inci,Targets, &incj);
+			p						= p*beta[0];
+			q						= q*beta[0];
+			Alpha[i]				= p*p/(q*q-p);
+			
+			if(Alpha[i]< 0) Alpha[i]				= init_alpha_max;
+			if(Alpha[i]> init_alpha_max) Alpha[i]				= init_alpha_max;
+		}
+		Free(projection);
+	}//of // is empty
+
+	int IsUsed					= 0;
+    kk							= 0;
+    for(i=0;i<M_full;i++)
+    {
+        IsUsed					= 0;
+        for(j=0;j<M;j++)
+        {
+            //index   = Used[j];
+            if ((i+1)==Used[j])     IsUsed  = 1;
+        }
+        if(IsUsed==0)     
+        {
+            Unused[kk]		= (i+ 1);
+            kk				= kk + 1;
+        }
+    }
+}
+
+
 void fEBInitializationGmNeEN(double *Alpha, double *PHI, int *Used, int *Unused, double *BASIS, 
 			double *Targets, double *Scales, int * initial, int N, int *m, int K, double *beta)
 {
@@ -720,7 +976,7 @@ void fEBInitializationGmNeEN(double *Alpha, double *PHI, int *Used, int *Unused,
 			readPtr1 = &BASIS[i*N];
 			proj = F77_CALL(ddot)(&N, readPtr1, &inci,Targets, &incj);
             proj			= proj/Scales[i];
-            if(fabs(proj) > fabs(proj_ini))
+            if(fabs(proj) < fabs(proj_ini))
             {
                 proj_ini    = proj;
                 loc1        = i;
@@ -808,6 +1064,8 @@ void fEBInitializationGmNeEN(double *Alpha, double *PHI, int *Used, int *Unused,
         }
     }
 }
+
+
 void LinearSolverGmNeEN(double * a, double *logout, int N, int M,double *output)
 {
 	const int nrhs		= 1;
@@ -841,7 +1099,7 @@ void LinearSolverGmNeEN(double * a, double *logout, int N, int M,double *output)
  /// ***********************************************************************************************   
 
 
-void CacheBPGmNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
+void CacheBPGmNeEN(double **BASIS_PHI, double *BASIS_Targets, double *BASIS, double *PHI,
 				double *Targets, double *scales,int N,int K,int M,int M_full)
 {
 	double	zTargets;
@@ -864,7 +1122,8 @@ void CacheBPGmNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, doub
 				cache2[h*M+l]		= PHI[l*N+h] * BASIS[i*N+ h];
 				z2[l]				= z2[l] + cache2[h*M+l];
 			}
-			BASIS_PHI[l*M_full+i]	= z2[l]/scales[i];
+			//BASIS_PHI[l*M_full+i]	= z2[l]/scales[i];
+			BASIS_PHI[l][i]	= (z2[l]/scales[i]);
 		}
 
 		zTargets					= 0;
@@ -906,12 +1165,12 @@ void CacheBPGmNeEN(double *BASIS_PHI, double *BASIS_Targets, double *BASIS, doub
     //Targets: nx1
 
 void fEBLinearFullStatGmNeEN(double *beta, double * SIGMA, double *H, double *S_in, double * Q_in, double * S_out, 
-				double * Q_out,   double *BASIS, double * Scales, double *PHI, double *BASIS_PHI,
+				double * Q_out,   double *BASIS, double * Scales, double *PHI, double **BASIS_PHI,
 				double *BASIS_Targets, double * Targets, int * Used, double *Alpha, double * Mu, 
 				 double *gamma,int *n, int *m, int* kdim, int *iteration,int *i_iter)
 {
     //basis dimension
-    int N,K,M,i;
+    int N,K,M,i,j,p;
    	N					= *n;			// row number
     K					= *kdim;		// column number
 	int M_full;
@@ -921,7 +1180,7 @@ void fEBLinearFullStatGmNeEN(double *beta, double * SIGMA, double *H, double *S_
 	//lapack
 	int inci 			=1;
 	int incj 			=1;
-	double *readPtr1;
+	//double *readPtr1;
 	double a_blas 		= 1;
 	double b_blas 		= 1;
 
@@ -989,25 +1248,28 @@ void fEBLinearFullStatGmNeEN(double *beta, double * SIGMA, double *H, double *S_
 
     for(i=0; i<M_full; i++)
     {
-		//for(j=0;j<M;j++)
-		//{
-		//	BPvector[j]			= 0;
+		for(j=0;j<M;j++)
+		{
+			BPvector[j]			= 0;
 		//	for(p=0;p<M;p++)	BPvector[j]		= BPvector[j] + BASIS_PHI[p*M_full + i]*SIGMA[j*M+p]; 
-		//}
-		readPtr1 	= &BASIS_PHI[i];
-		b_blas 		= 0;
-		F77_CALL(dgemv)(&transa, &M, &M,&a_blas, SIGMA, &M, readPtr1, &M_full, &b_blas,BPvector, &incj);
+		for(p=0;p<M;p++)	BPvector[j]		= BPvector[j] + BASIS_PHI[p][i]*SIGMA[j*M+p]; 
+		}
+		//readPtr1 	= &BASIS_PHI[i];
+		//b_blas 		= 0;
+		//F77_CALL(dgemv)(&transa, &M, &M,&a_blas, SIGMA, &M, readPtr1, &M_full, &b_blas,BPvector, &incj);
 		
         tempSum					= 0;
 		//for(j=0;j<M;j++)		tempSum			= tempSum + BPvector[j]*BASIS_PHI[j*M_full+i];
-		readPtr1 	= &BASIS_PHI[i];
-		tempSum		= F77_CALL(ddot)(&M, BPvector, &inci,readPtr1, &M_full);
+		for(j=0;j<M;j++)		tempSum			= tempSum + BPvector[j]*BASIS_PHI[j][i];
+		//readPtr1 	= &BASIS_PHI[i];
+		//tempSum		= F77_CALL(ddot)(&M, BPvector, &inci,readPtr1, &M_full);
 		
 		S_in[i]					= beta[0] - beta[0]*tempSum*beta[0];
 		tempBPMu				= 0;
 		//for(p=0;p<M;p++) tempBPMu = tempBPMu + BASIS_PHI[p*M_full + i] *Mu[p];
-		readPtr1 	= &BASIS_PHI[i];
-		tempBPMu 	= F77_CALL(ddot)(&M, Mu, &inci,readPtr1, &M_full);	
+		for(p=0;p<M;p++) tempBPMu = tempBPMu + BASIS_PHI[p][i] *Mu[p];
+		//readPtr1 	= &BASIS_PHI[i];
+		//tempBPMu 	= F77_CALL(ddot)(&M, Mu, &inci,readPtr1, &M_full);	
 		
 		Q_in[i]					= beta[0]*(BASIS_Targets[i] - tempBPMu);
         //S_out[i]         		= S_in[i];
@@ -1067,7 +1329,8 @@ void MatrixInverseGmNeEN(double * a,int N)
 //***************************************************************************************************
 void fEBDeltaMLGmNeEN(double *DeltaML, int *Action, double *AlphaRoot, int *anyToDelete,
 				int *Used, int * Unused, double * S_out, double * Q_out, double *Alpha,
-				double *a_lambda, double *b_Alpha, int N_used, int N_unused)
+				double *a_lambda, double *b_Alpha, int N_used, int N_unused,
+				double *deltaLogMarginal, int *nu)//Block Coordinate Ascent Algorithm Parameters;
 {
     //basis dimension
     int M_full,i,index;
@@ -1084,10 +1347,16 @@ anyToDelete[0] = 0;
 	lambda1 							= *a_lambda * (b_Alpha[0]);
 	lambda2 							= *a_lambda * (1 - b_Alpha[0]);
     //int   anyToDelete					= 0;
-    int   anyToAdd						= 0;
-    const int CNPriorityAddition		= 0;
-    const int CNPriorityDeletion		= 1;
+    //int   anyToAdd						= 0;
+    //const int CNPriorityAddition		= 0;
+    //const int CNPriorityDeletion		= 1;
 	//Rprintf("Inside fEBDeltaMLBmNeEN: a: %f, b %f \n",a, b);
+	
+	//Block Coordinate Ascent Algorithm Parameters;
+	for(i=0;i<M_full;i++) Action[i] = -10;
+	double deltaMLmax = 0;
+	int 	max = 0;
+	
     for(i=0;i<N_used;i++)
     {
         //anyToDelete				= false;
@@ -1126,6 +1395,14 @@ anyToDelete[0] = 0;
             logL                = (log(oldAlpha/(oldAlpha+S_out[index] + lambda2)) + pow(Q_out[index],2)/(oldAlpha + S_out[index] + lambda2))*0.5 - lambda1/oldAlpha;
             DeltaML[index]      = - logL;
         }
+		
+		//Block Coordinate Ascent Algorithm Parameters;
+		if(DeltaML[index]>deltaMLmax)
+		{
+				max			= index;
+				deltaMLmax 	= DeltaML[index];
+		}
+		
     }
     //ADDITION
     for(i=0;i<N_unused;i++)
@@ -1150,10 +1427,44 @@ anyToDelete[0] = 0;
             }
         }
         //case 2
-        //case 3        
+        //case 3  
+		//Block Coordinate Ascent Algorithm Parameters;
+		if(DeltaML[index]>deltaMLmax)
+		{
+				max			= index;
+				deltaMLmax 	= DeltaML[index];
+		} 		
     }
-        
-	//
+    
+	//Block Coordinate Ascent Algorithm Parameters; 
+	// only one action for a block
+	int selectedAction = Action[max];
+	
+	/*
+	if(i_iter <=(1*N_used)) // re-estimate
+	{
+		
+		for(i=0;i<M_full;i++)
+		{
+			if(Action[i] !=ACTION_REESTIMATE) DeltaML[i] = 0;
+		}
+	}else
+	{
+		for(i=0;i<M_full;i++)
+		{
+			if(Action[i] !=selectedAction) DeltaML[i] = 0;
+		}
+	}
+	*/
+		for(i=0;i<M_full;i++)
+		{
+			if(Action[i] !=selectedAction) DeltaML[i] = 0;
+		}
+
+deltaLogMarginal[0] = deltaMLmax;
+nu[0] 	= max;
+    
+	/*
     if((anyToAdd==1 && CNPriorityAddition==1) || (anyToDelete[0]==1 && CNPriorityDeletion==1))
     {
         for(i=0;i<M_full;i++)
@@ -1167,11 +1478,13 @@ anyToDelete[0] = 0;
                     if(anyToDelete[0] ==1 && CNPriorityDeletion==1 && CNPriorityAddition!=1) DeltaML[i] = 0;
             }
         }
-    }    
+    }  
+
+	*/
 }
 
 //888888888888888888888888888888888888888888888888888888888888888888888888888888888888
-int ActionAddGmNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
+int ActionAddGmNeEN(double **BASIS_PHI, double* BASIS, double*scales, double*PHI, double*Phi,
 			double *beta, double* Alpha, double newAlpha, double*SIGMA, double*Mu, double*S_in,
 			double*Q_in, int nu, double*SIGMANEW, int M_full,int N, int K, int M)
 {
@@ -1289,17 +1602,18 @@ int ActionAddGmNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI,
 	{
 		temp				= 0;
 		//for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j*M_full + i]*tmpp[j];
-		readPtr1 			= &BASIS_PHI[i];
-		temp 				= F77_CALL(ddot)(&M, readPtr1, &M_full,tmpp,&incj); 
+		for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j][i]*tmpp[j];
+		//readPtr1 			= &BASIS_PHI[i];
+		//temp 				= F77_CALL(ddot)(&M, readPtr1, &M_full,tmpp,&incj); 
 		
 		mCi[i]				= BASIS_B_Phi[i] -  beta[0]*temp;
-		BASIS_PHI[M*M_full + i]					= BASIS_Phi[i];
+		//BASIS_PHI[M*M_full + i]					= BASIS_Phi[i];
 		S_in[i]				= S_in[i] - mCi[i]*mCi[i]*s_ii;
 		Q_in[i]				= Q_in[i] - mu_i*mCi[i];
 	}
-	
+	BASIS_PHI[M]=BASIS_Phi;
 	int UPDATE_REQUIRED		= 1;
-	Free(BASIS_Phi);
+	//Free(BASIS_Phi);
 	Free(BASIS_B_Phi);	
 	Free(mCi);
 	Free(z);	
@@ -1309,17 +1623,19 @@ int ActionAddGmNeEN(double *BASIS_PHI, double* BASIS, double*scales, double*PHI,
 	
 	return  UPDATE_REQUIRED; 
  }
-int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double*BASIS_PHI,
+
+int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, double**BASIS_PHI,
 		double*Mu, double*S_in, double*Q_in, double *beta, int jj, int N, int M, int M_full)
  {
 	 int i,j;
 	int index				= M - 1;
-		//lapack
+	//lapack
 	int inci =1;
 	int incj =1;
 	double *readPtr1, *readPtr2;
 	//lapack end
 	
+	
 	//
 	Alpha[jj]				= Alpha[index];           //Alpha: M -> M - 1
             
@@ -1332,10 +1648,10 @@ int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, dou
     int Mujj;        
 	Mujj					= Mu[jj];
 	for(i=0;i<M;i++)  Mu[i] = Mu[i] - Mujj*SIGMA[jj*M +i]/SIGMA[jj*M + jj];
-            
-	//
-	Mu[jj]					= Mu[index];
 
+	//
+	Mu[jj]					= Mu[index];        
+	//------------------------------------------------------------------------------
 	//BLOCK MIGRATION OF SIGMANEW ------------------JUN142013 -------IN EBELASTICNET
 	double *tempSIGMA = (double *) Calloc((M*M),double);
 	for(i=0;i<M;i++)
@@ -1346,7 +1662,7 @@ int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, dou
 	{
 		for(j=0;j<index;j++) SIGMANEW[j*index + i]	= tempSIGMA[j*M + i];
 	}
-	if(jj!=index)
+	if(jj != index)// incase the one to be deleted is the last column.
 	{
 	//step 1: last column
 	readPtr1 = &SIGMANEW[jj*index];
@@ -1381,22 +1697,26 @@ int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, dou
 	
 	//BLOCK MIGRATION OF SIGMANEW ------------------JUN142013 -------IN EBELASTICNET	
 	
+	
 	double temp;
 	for(i=0;i<M_full;i++)
 	{
 		temp				= 0;
-		for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
+		//for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j*M_full + i]*SIGMA[jj*M + j];
+		for(j=0;j<M;j++)	temp				= temp + BASIS_PHI[j][i]*SIGMA[jj*M + j];
 		S_in[i]				= S_in[i] +  pow(beta[0]*temp,2)/SIGMA[jj*M + jj];
 		Q_in[i]				= Q_in[i] +  beta[0]*temp *Mujj /SIGMA[jj*M + jj];
 		//jPm = beta*temp
 	}
-	
+	double *ptr;
+	ptr=BASIS_PHI[jj];
+	BASIS_PHI[jj]=BASIS_PHI[index];
+	BASIS_PHI[index]=ptr;
 	//BASIS_PHI(:,jj) = [];
 	//for(i=0;i<M_full;i++) BASIS_PHI[jj*M_full + i]	= BASIS_PHI[index*M_full+i];
-	readPtr1 = &BASIS_PHI[jj*M_full];
-	readPtr2 = &BASIS_PHI[index*M_full];
-	F77_CALL(dcopy)(&M_full,readPtr2,&inci,readPtr1,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
-	
+	//readPtr1 = &BASIS_PHI[jj*M_full];
+	//readPtr2 = &BASIS_PHI[index*M_full];
+	//F77_CALL(dcopy)(&M_full,readPtr2,&inci,readPtr1,&incj);  //dcopy(n, x, incx, y, incy) ---> y = x
 	
 	int UPDATE_REQUIRED		=1;
 	Free(tempSIGMA);
@@ -1404,6 +1724,7 @@ int ActionDelGmNeEN(double*PHI, double*Alpha, double*SIGMA, double*SIGMANEW, dou
 }//end of ACTION_DELETE
 
 
+
 double varTargetsGmNeEN(double* Targets,int N)
 {
 	int i;
